from utils.application import mysql
from flask import jsonify, request
from threading import Lock

# Create a lock to avoid a race condition when getting and setting stock quantity
mutex_lock = Lock()

def post_create_order():
    # Get user from logged in cookie
    userId = request.cookies.get('logged_in')
    if userId is None:
        return jsonify({"message": "You are not logged in"}), 401

    data = request.json

    # Products is array of {product_id, quantity}

    # Example body:

        # "products": [
        #   {
        #      "productid": 1,
        #      "quantity": 2 
        #   },
        #   ...
        # ],

    products = data["products"]

    productIds = []
    for product in products:
        productIds.append(product["product_id"])

    if (len(products) <= 0):
        return jsonify({"message": "No products in order"}), 400
    
    # Acquire the mutex lock
    mutex_lock.acquire()

    try:
        # Check if all products exist and get their price
        cur = mysql.connection.cursor()
        cur.execute('''SELECT price, stock_quantity FROM product WHERE product_id IN %s''', (productIds,))
        productsData = cur.fetchall()
        cur.close()

        if len(productsData) != len(products):
            return jsonify({"message": "A product you are attempting to purchase does not exist"}), 400

        # Check if all products are in stock
        for i in range(len(productsData)):
            if productsData[i][1] < products[i]["quantity"]:
                return jsonify({"message": "A product you are attempting to purchase is out of stock"}), 400

        totalAmount = 0
        for i in range(len(productsData)):
            totalAmount += productsData[i][0] * products[i]["quantity"]

        # Create user_order, it has order_date, total_amount, status, user_id
        cur = mysql.connection.cursor()
        cur.execute('''INSERT INTO user_order (order_date, total_amount, status, user_id) VALUES (NOW(), %s, "Pending", %s)''', (totalAmount, userId))
        mysql.connection.commit()
        cur.close()

        # Get the order_id
        cur = mysql.connection.cursor()
        cur.execute('''SELECT order_id FROM user_order WHERE user_id = %s ORDER BY order_id DESC LIMIT 1''', (userId,))
        order_id = cur.fetchall()[0][0]
        cur.close()

        # Create order_item for each product
        for i in range(len(products)):
            cur = mysql.connection.cursor()
            cur.execute('''INSERT INTO order_item (quantity, product_id, order_id) VALUES (%s, %s, %s)''', (products[i]["quantity"], products[i]["product_id"], order_id))
            mysql.connection.commit()
            cur.close()

        # Reduce stock_quantity for each product
        for i in range(len(products)):
            cur = mysql.connection.cursor()
            cur.execute('''UPDATE product SET stock_quantity = stock_quantity - %s WHERE product_id = %s''', (products[i]["quantity"], products[i]["product_id"]))
            mysql.connection.commit()
            cur.close()

    finally:
        # Release the lock even if the try-block fails
        mutex_lock.release()

    return jsonify({"message": "Order placed"}), 201

def pay_order():
    # Get user from logged in cookie
    userId = request.cookies.get('logged_in')
    if userId is None:
        return jsonify({"message": "You are not logged in"}), 401

    # Example body:

    # {
    #   "order_id": 6,
    #   "payment_method": "Vipps"
    # }

    data = request.json

    order_id = data['order_id']
    payment_method = data['payment_method']

    # Get the user_id of the user with the email from the user table
    # Check that the user_id logged in is the same as the user on the order
    # Get the order status to make sure the order is not already paid

    cur = mysql.connection.cursor()
    cur.execute('''SELECT user_order.status, user_order.total_amount, user_order.user_id
                FROM user_order
                WHERE order_id = %s''', (order_id,))
    order_info = cur.fetchall()
    cur.close()

    # Check we have one and only one order
    if len(order_info) != 1:
        return jsonify({"message": "Not a valid order id"}), 400

    # Destructure the query into variables
    status, amount, user_id = order_info[0]

    # Check email is the same as logged in email
    if userId != str(user_id):
        return jsonify({"message": "This order is not for the logged in user"}), 400

    # Check order is not already paid
    if status != "Pending":
        return jsonify({"message": "Order is already paid"}), 400

    # Make sure correct payment method
    if payment_method != "Vipps":
        return jsonify({"message": "Invalid payment method"}), 400

    # Pay the order
    payment = vipps(amount, order_id)

    if not payment:
        return jsonify({"message": "Payment failed"}), 400

    # Update the status of the order to paid
    cur = mysql.connection.cursor()
    cur.execute('''UPDATE user_order SET status = 'Paid' WHERE user_order.order_id = %s''', (order_id,))
    mysql.connection.commit()
    cur.close()

    return jsonify({"message": "Order paid"}), 200

# Dummy payment method
def vipps(amount, order_id):
    return True

# Get all orders for a user
def get_all_orders():
    # Get user from logged in cookie
    userId = request.cookies.get('logged_in')

    # Check if user is logged in
    if userId is None:
        return jsonify({"message": "You are not logged in"}), 401

    # Get orders from database
    cur = mysql.connection.cursor()
    cur.execute('''SELECT user_order.order_id, user_order.order_date, user_order.total_amount, user_order.status FROM user_order WHERE user_id = %s''', (userId,))
    ordersDb = cur.fetchall()
    cur.close()

    # Format orders
    orders = []
    for order in ordersDb:
        orders.append({
            "order_id": order[0],
            "order_date": order[1].isoformat(),
            "total_amount": order[2],
            "status": order[3]
        })

    # Return orders
    return jsonify(orders), 200